@@ -26,11 +26,28 @@
 extern "C" {
 #endif
 
+#define OS_RELEASE_FILE_DIR "tests/release_files"
+
 
 #ifndef BINARY_DIR
 # define BINARY_DIR "tests/binary"
 #endif
 
+#ifndef CERT_TEST_DIR
+# define CERT_TEST_DIR "tests/x509gen"
+#endif
+
+#define CERT_CA                  CERT_TEST_DIR "/ca.pem"
+#define CERT_CRL                 CERT_TEST_DIR "/crl.pem"
+#define CERT_SERVER              CERT_TEST_DIR "/server.pem" /* 127.0.0.1 & localhost */
+#define CERT_CLIENT              CERT_TEST_DIR "/client.pem"
+#define CERT_ALTNAME             CERT_TEST_DIR "/altname.pem" /* alternative.mongodb.org */
+#define CERT_WILD                CERT_TEST_DIR "/wild.pem" /* *.mongodb.org */
+#define CERT_COMMONNAME          CERT_TEST_DIR "/commonName.pem" /* 127.0.0.1 & localhost */
+#define CERT_EXPIRED             CERT_TEST_DIR "/expired.pem" /* 127.0.0.1 & localhost */
+#define CERT_PASSWORD            "qwerty"
+#define CERT_PASSWORD_PROTECTED  CERT_TEST_DIR "/password_protected.pem"
+
 
 #ifdef ASSERT
 # undef ASSERT
@@ -41,6 +58,7 @@ extern "C" {
 #ifdef ASSERT_OR_PRINT
 # undef ASSERT_OR_PRINT
 #endif
+
 #define ASSERT_OR_PRINT(_statement, _err) \
    do { \
       if (! (_statement)) { \
@@ -52,6 +70,45 @@ extern "C" {
       } \
    } while (0)
 
+#define ASSERT_CURSOR_NEXT(_cursor, _doc) \
+   do { \
+      bson_error_t _err; \
+      if (! mongoc_cursor_next ((_cursor), (_doc))) { \
+         if (mongoc_cursor_error ((_cursor), &_err)) { \
+            fprintf(stderr, "FAIL:%s:%d  %s()\n  %s\n\n", \
+                            __FILE__, __LINE__, BSON_FUNC, \
+                            _err.message); \
+          } else { \
+             fprintf(stderr, "FAIL:%s:%d  %s()\n  %s\n\n", \
+                            __FILE__, __LINE__, BSON_FUNC, \
+                            "empty cursor"); \
+          } \
+          fflush(stderr); \
+          abort(); \
+      } \
+   } while (0)
+
+
+#define ASSERT_CURSOR_DONE(_cursor) \
+   do { \
+      bson_error_t _err; \
+      const bson_t *_doc; \
+      if (mongoc_cursor_next ((_cursor), &_doc)) { \
+         fprintf(stderr, "FAIL:%s:%d  %s()\n  %s\n\n", \
+                         __FILE__, __LINE__, BSON_FUNC, \
+                         "non-empty cursor"); \
+         fflush(stderr); \
+         abort(); \
+      } \
+      if (mongoc_cursor_error ((_cursor), &_err)) { \
+         fprintf(stderr, "FAIL:%s:%d  %s()\n  %s\n\n", \
+                         __FILE__, __LINE__, BSON_FUNC, \
+                         _err.message); \
+         fflush(stderr); \
+         abort(); \
+      } \
+   } while (0)
+
 
 #define ASSERT_CMPINT_HELPER(a, eq, b, fmt) \
    do { \
@@ -72,9 +129,11 @@ extern "C" {
 #define ASSERT_CMPINT32(a, eq, b)  ASSERT_CMPINT_HELPER(a, eq, b, PRId32)
 #define ASSERT_CMPINT64(a, eq, b)  ASSERT_CMPINT_HELPER(a, eq, b, PRId64)
 #define ASSERT_CMPUINT16(a, eq, b)   ASSERT_CMPINT_HELPER(a, eq, b, "hu")
+#define ASSERT_CMPUINT32(a, eq, b)   ASSERT_CMPINT_HELPER(a, eq, b, PRIu32)
 #define ASSERT_CMPUINT64(a, eq, b) ASSERT_CMPINT_HELPER(a, eq, b, PRIu64)
 #define ASSERT_CMPSIZE_T(a, eq, b) ASSERT_CMPINT_HELPER(a, eq, b, "zd")
 #define ASSERT_CMPSSIZE_T(a, eq, b) ASSERT_CMPINT_HELPER(a, eq, b, "zx")
+#define ASSERT_CMPVOID(a, eq, b) ASSERT_CMPINT_HELPER(a, eq, b, "p")
 
 #define ASSERT_MEMCMP(a, b, n) \
    do { \
@@ -172,6 +231,28 @@ extern "C" {
       ASSERT_CONTAINS (error.message, _message); \
    } while (0);
 
+#define ASSERT_CAPTURED_LOG(_info, _level, _msg) \
+   do { \
+      if (!has_captured_log (_level, _msg)) { \
+         fprintf (stderr, \
+                  "%s:%d %s(): testing %s didn't log \"%s\"\n", \
+                  __FILE__, __LINE__, BSON_FUNC, _info, _msg); \
+         print_captured_logs ("\t"); \
+         abort (); \
+      } \
+   } while (0);
+
+#define ASSERT_NO_CAPTURED_LOGS(_info) \
+   do { \
+      if (has_captured_logs ()) { \
+         fprintf (stderr, \
+                  "%s:%d %s(): testing %s shouldn't have logged:\n", \
+                  __FILE__, __LINE__, BSON_FUNC, _info); \
+         print_captured_logs ("\t"); \
+         abort (); \
+      } \
+   } while (0);
+
 #define ASSERT_HAS_FIELD(_bson, _field) \
    do { \
       if (!bson_has_field ((_bson), (_field))) { \
@@ -191,6 +272,40 @@ extern "C" {
    } while (0)
 
 
+#ifdef _WIN32
+# define gettestpid _getpid
+#else
+# define gettestpid getpid
+#endif
+
+#if (__GNUC__ > 4) || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6)
+# define BEGIN_IGNORE_DEPRECATIONS \
+   _Pragma ("GCC diagnostic push") \
+   _Pragma ("GCC diagnostic ignored \"-Wdeprecated-declarations\"")
+# define END_IGNORE_DEPRECATIONS \
+   _Pragma ("GCC diagnostic pop")
+#elif defined(__clang__)
+# define BEGIN_IGNORE_DEPRECATIONS \
+   _Pragma ("clang diagnostic push") \
+   _Pragma ("clang diagnostic ignored \"-Wdeprecated-declarations\"")
+# define END_IGNORE_DEPRECATIONS \
+   _Pragma ("clang diagnostic pop")
+#else
+# define BEGIN_IGNORE_DEPRECATIONS
+# define END_IGNORE_DEPRECATIONS
+#endif
+
+#define ASSERT_OR_PRINT_ERRNO(_statement, _errcode) \
+   do { \
+      if (! (_statement)) { \
+         fprintf(stderr, "FAIL:%s:%d  %s()\n  %s\n  Failed with error code: %d (%s)\n\n", \
+                         __FILE__, __LINE__, BSON_FUNC, \
+                         #_statement, _errcode, strerror(_errcode)); \
+         fflush(stderr); \
+         abort(); \
+      } \
+   } while (0)
+
 #define MAX_TEST_NAME_LENGTH 500
 
 
@@ -222,6 +337,7 @@ struct _TestSuite
    Test *tests;
    FILE *outfile;
    int flags;
+   int silent;
 };
 
 
@@ -232,6 +348,10 @@ void TestSuite_Init    (TestSuite *suite,
 void TestSuite_Add     (TestSuite *suite,
                         const char *name,
                         TestFunc func);
+int TestSuite_CheckLive (void);
+void TestSuite_AddLive (TestSuite *suite,
+                        const char *name,
+                        TestFunc func);
 void TestSuite_AddWC   (TestSuite *suite,
                         const char *name,
                         TestFuncWC func,
