@@ -17,6 +17,7 @@
 
 #include <bson.h>
 #include <mongoc.h>
+#include <mongoc-host-list-private.h>
 
 #include "mongoc-server-description.h"
 #include "mongoc-server-description-private.h"
@@ -25,35 +26,47 @@
 #include "mongoc-uri-private.h"
 #include "mongoc-util-private.h"
 
-#include "mongoc-tests.h"
 #include "TestSuite.h"
 #include "test-conveniences.h"
 #include "test-libmongoc.h"
 
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 
 extern void test_array_install                   (TestSuite *suite);
 extern void test_async_install                   (TestSuite *suite);
 extern void test_buffer_install                  (TestSuite *suite);
 extern void test_bulk_install                    (TestSuite *suite);
 extern void test_client_install                  (TestSuite *suite);
+#ifdef MONGOC_EXPERIMENTAL_FEATURES
+extern void test_client_max_staleness_install    (TestSuite *suite);
+#endif
 extern void test_client_pool_install             (TestSuite *suite);
 extern void test_cluster_install                 (TestSuite *suite);
 extern void test_collection_install              (TestSuite *suite);
 extern void test_collection_find_install         (TestSuite *suite);
+extern void test_command_monitoring_install      (TestSuite *suite);
 extern void test_cursor_install                  (TestSuite *suite);
 extern void test_database_install                (TestSuite *suite);
+extern void test_error_install                   (TestSuite *suite);
 extern void test_exhaust_install                 (TestSuite *suite);
 extern void test_find_and_modify_install         (TestSuite *suite);
 extern void test_gridfs_file_page_install        (TestSuite *suite);
 extern void test_gridfs_install                  (TestSuite *suite);
+#ifdef MONGOC_EXPERIMENTAL_FEATURES
+extern void test_linux_distro_scanner_install    (TestSuite *suite);
+#endif
 extern void test_list_install                    (TestSuite *suite);
 extern void test_log_install                     (TestSuite *suite);
 extern void test_matcher_install                 (TestSuite *suite);
+#ifdef MONGOC_EXPERIMENTAL_FEATURES
+extern void test_metadata_install                (TestSuite *suite);
+#endif
 extern void test_queue_install                   (TestSuite *suite);
 extern void test_read_prefs_install              (TestSuite *suite);
 extern void test_rpc_install                     (TestSuite *suite);
 extern void test_sdam_install                    (TestSuite *suite);
-extern void test_sasl_install                    (TestSuite *suite);
 extern void test_server_selection_install        (TestSuite *suite);
 extern void test_server_selection_errors_install (TestSuite *suite);
 extern void test_set_install                     (TestSuite *suite);
@@ -65,46 +78,182 @@ extern void test_topology_reconcile_install      (TestSuite *suite);
 extern void test_topology_scanner_install        (TestSuite *suite);
 extern void test_uri_install                     (TestSuite *suite);
 extern void test_usleep_install                  (TestSuite *suite);
+extern void test_util_install                    (TestSuite *suite);
 extern void test_version_install                 (TestSuite *suite);
 extern void test_write_command_install           (TestSuite *suite);
 extern void test_write_concern_install           (TestSuite *suite);
 #ifdef MONGOC_ENABLE_SSL
-extern void test_x509_install                    (TestSuite *suite);
 extern void test_stream_tls_install              (TestSuite *suite);
+extern void test_x509_install                    (TestSuite *suite);
 extern void test_stream_tls_error_install        (TestSuite *suite);
 #endif
+#ifdef MONGOC_ENABLE_SASL
+extern void test_sasl_install                    (TestSuite *suite);
+#endif
 
 
-static int gSuppressCount;
+typedef struct {
+   mongoc_log_level_t  level;
+   char               *msg;
+} log_entry_t;
+
+static mongoc_mutex_t captured_logs_mutex;
+static mongoc_array_t captured_logs;
+static bool capturing_logs;
 #ifdef MONGOC_ENABLE_SSL
 static mongoc_ssl_opt_t gSSLOptions;
 #endif
 
 
-void
-suppress_one_message (void)
+static log_entry_t *
+log_entry_create (mongoc_log_level_t  level,
+                  const char         *msg)
 {
-   gSuppressCount++;
+   log_entry_t *log_entry;
+
+   log_entry = bson_malloc (sizeof (log_entry_t));
+   log_entry->level = level;
+   log_entry->msg = bson_strdup (msg);
+
+   return log_entry;
 }
 
 
+static void
+log_entry_destroy (log_entry_t *log_entry)
+{
+   bson_free (log_entry->msg);
+   bson_free (log_entry);
+}
+
+
+void
+capture_logs (bool capture)
+{
+   capturing_logs = capture;
+   clear_captured_logs ();
+}
+
+
+void
+clear_captured_logs (void)
+{
+   size_t i;
+   log_entry_t *log_entry;
+
+   mongoc_mutex_lock (&captured_logs_mutex);
+   for (i = 0; i < captured_logs.len; i++) {
+      log_entry = _mongoc_array_index (&captured_logs, log_entry_t *, i);
+      log_entry_destroy (log_entry);
+   }
+
+   captured_logs.len = 0;
+   mongoc_mutex_unlock (&captured_logs_mutex);
+}
+
+
+bool
+has_captured_log (mongoc_log_level_t  level,
+                  const char         *msg)
+{
+   size_t i;
+   log_entry_t *log_entry;
+
+   mongoc_mutex_lock (&captured_logs_mutex);
+
+   for (i = 0; i < captured_logs.len; i++) {
+      log_entry = _mongoc_array_index (&captured_logs, log_entry_t *, i);
+      if (level == log_entry->level && strstr (log_entry->msg, msg)) {
+         mongoc_mutex_unlock (&captured_logs_mutex);
+         return true;
+      }
+   }
+
+   mongoc_mutex_unlock (&captured_logs_mutex);
+
+   return false;
+}
+
+
+bool
+has_captured_logs (void)
+{
+   bool ret;
+
+   mongoc_mutex_lock (&captured_logs_mutex);
+   ret = 0 != captured_logs.len;
+   mongoc_mutex_unlock (&captured_logs_mutex);
+
+   return ret;
+}
+
+
+void
+print_captured_logs (const char *prefix)
+{
+   size_t i;
+   log_entry_t *log_entry;
+
+   mongoc_mutex_lock (&captured_logs_mutex);
+   for (i = 0; i < captured_logs.len; i++) {
+      log_entry = _mongoc_array_index (&captured_logs, log_entry_t *, i);
+      if (prefix) {
+         fprintf (stderr, "%s%s %s\n",
+                  prefix,
+                  mongoc_log_level_str (log_entry->level),
+                  log_entry->msg);
+      } else {
+         fprintf (stderr, "%s %s\n",
+                  mongoc_log_level_str (log_entry->level),
+                  log_entry->msg);
+      }
+   }
+   mongoc_mutex_unlock (&captured_logs_mutex);
+}
+
+
+#define DEFAULT_FUTURE_TIMEOUT_MS 10 * 1000
+
+int64_t
+get_future_timeout_ms ()
+{
+    return test_framework_getenv_int64 ("MONGOC_TEST_FUTURE_TIMEOUT_MS",
+                                        DEFAULT_FUTURE_TIMEOUT_MS);
+}
+
 static void
 log_handler (mongoc_log_level_t  log_level,
              const char         *log_domain,
              const char         *message,
              void               *user_data)
 {
+   TestSuite *suite;
+   log_entry_t *log_entry;
+
+   suite = (TestSuite *) user_data;
+
    if (log_level < MONGOC_LOG_LEVEL_INFO) {
-      if (gSuppressCount) {
-         gSuppressCount--;
+      if (capturing_logs) {
+         log_entry = log_entry_create (log_level, message);
+         mongoc_mutex_lock (&captured_logs_mutex);
+         _mongoc_array_append_val (&captured_logs, log_entry);
+         mongoc_mutex_unlock (&captured_logs_mutex);
          return;
       }
-      mongoc_log_default_handler (log_level, log_domain, message, NULL);
+
+      if (!suite->silent) {
+         mongoc_log_default_handler (log_level, log_domain, message, NULL);
+      }
    }
 }
 
 
-char MONGOC_TEST_UNIQUE [32];
+mongoc_database_t *
+get_test_database (mongoc_client_t *client)
+{
+   return mongoc_client_get_database (client, "test");
+}
+
 
 char *
 gen_collection_name (const char *str)
@@ -116,6 +265,22 @@ gen_collection_name (const char *str)
 
 }
 
+
+mongoc_collection_t *
+get_test_collection (mongoc_client_t *client,
+                     const char      *prefix)
+{
+   mongoc_collection_t *ret;
+   char *str;
+
+   str = gen_collection_name (prefix);
+   ret = mongoc_client_get_collection (client, "test", str);
+   bson_free (str);
+
+   return ret;
+}
+
+
 /*
  *--------------------------------------------------------------------------
  *
@@ -233,12 +398,27 @@ test_framework_getenv_int64 (const char *name,
 }
 
 
+static char *
+test_framework_get_unix_domain_socket_path (void)
+{
+   char *path = test_framework_getenv ("MONGOC_TEST_UNIX_DOMAIN_SOCKET");
+
+   if (path) {
+      return path;
+   }
+
+   return bson_strdup_printf ("/tmp/mongodb-%d.sock",
+         test_framework_get_port());
+}
+
+
 /*
  *--------------------------------------------------------------------------
  *
- * test_framework_get_unix_domain_socket_path --
+ * test_framework_get_unix_domain_socket_path_escaped --
  *
- *       Get the path to Unix Domain Socket .sock of the test MongoDB server.
+ *       Get the path to Unix Domain Socket .sock of the test MongoDB server,
+ *       URI-escaped ("/" is replaced with "%2F").
  *
  * Returns:
  *       A string you must bson_free.
@@ -249,16 +429,54 @@ test_framework_getenv_int64 (const char *name,
  *--------------------------------------------------------------------------
  */
 char *
-test_framework_get_unix_domain_socket_path (void)
+test_framework_get_unix_domain_socket_path_escaped (void)
 {
-   char *path = test_framework_getenv ("MONGOC_TEST_UNIX_DOMAIN_SOCKET");
+   char *path = test_framework_get_unix_domain_socket_path (), *c = path;
+   bson_string_t *escaped = bson_string_new (NULL);
 
-   if (path) {
-      return path;
-   }
-   return bson_strdup_printf ("%%2Ftmp%%2Fmongodb-%d.sock",
-         test_framework_get_port());
+   /* Connection String Spec: "The host information cannot contain an unescaped
+    * slash ("/"), if it does then an exception MUST be thrown informing users
+    * that paths must be URL encoded."
+    *
+    * Even though the C Driver does not currently enforce the spec, let's pass
+    * a correctly escaped URI.
+    */
+   do {
+      if (*c == '/') {
+         bson_string_append (escaped, "%2F");
+      } else {
+         bson_string_append_c (escaped, *c);
+      }
+   } while (*(++c));
+
+   bson_string_append_c (escaped, '\0');
+   bson_free (path);
+
+   return bson_string_free (escaped, false /* free_segment */);
 }
+
+static char *
+_uri_str_from_env (void)
+{
+   return test_framework_getenv ("MONGOC_TEST_URI");
+}
+
+static mongoc_uri_t *
+_uri_from_env (void)
+{
+   char *env_uri_str;
+   mongoc_uri_t *uri;
+
+   env_uri_str = _uri_str_from_env ();
+   if (env_uri_str) {
+      uri = mongoc_uri_new (env_uri_str);
+      bson_free (env_uri_str);
+      return uri;
+   }
+
+   return NULL;
+}
+
 /*
  *--------------------------------------------------------------------------
  *
@@ -277,7 +495,21 @@ test_framework_get_unix_domain_socket_path (void)
 char *
 test_framework_get_host (void)
 {
-   char *host = test_framework_getenv ("MONGOC_TEST_HOST");
+   mongoc_uri_t *env_uri;
+   const mongoc_host_list_t *hosts;
+   char *host;
+
+   /* MONGOC_TEST_URI takes precedence */
+   env_uri = _uri_from_env ();
+   if (env_uri) {
+      /* choose first host */
+      hosts = mongoc_uri_get_hosts (env_uri);
+      host = bson_strdup (hosts->host);
+      mongoc_uri_destroy (env_uri);
+      return host;
+   }
+
+   host = test_framework_getenv ("MONGOC_TEST_HOST");
 
    return host ? host : bson_strdup ("localhost");
 }
@@ -300,22 +532,60 @@ test_framework_get_host (void)
 uint16_t
 test_framework_get_port (void)
 {
-   char *port_str = test_framework_getenv ("MONGOC_TEST_PORT");
+   mongoc_uri_t *env_uri;
+   const mongoc_host_list_t *hosts;
+   char *port_str;
    unsigned long port = MONGOC_DEFAULT_PORT;
 
-   if (port_str && strlen (port_str)) {
-      port = strtoul (port_str, NULL, 10);
-      if (port == 0 || port > UINT16_MAX) {
-         /* parse err or port out of range -- mongod prohibits port 0 */
-         port = MONGOC_DEFAULT_PORT;
+   /* MONGOC_TEST_URI takes precedence */
+   env_uri = _uri_from_env ();
+   if (env_uri) {
+      /* choose first port */
+      hosts = mongoc_uri_get_hosts (env_uri);
+      port = hosts->port;
+      mongoc_uri_destroy (env_uri);
+   } else {
+      port_str = test_framework_getenv ("MONGOC_TEST_PORT");
+      if (port_str && strlen (port_str)) {
+         port = strtoul (port_str, NULL, 10);
+         if (port == 0 || port > UINT16_MAX) {
+            /* parse err or port out of range -- mongod prohibits port 0 */
+            port = MONGOC_DEFAULT_PORT;
+         }
       }
+
+      bson_free (port_str);
    }
 
-   bson_free (port_str);
-
    return (uint16_t) port;
 }
 
+/*
+ *--------------------------------------------------------------------------
+ *
+ * test_framework_get_host_list --
+ *
+ *       Get the single host and port of the test server (not actually a
+ *       list).
+ *
+ * Side effects:
+ *       None.
+ *
+ *--------------------------------------------------------------------------
+ */
+void
+test_framework_get_host_list (mongoc_host_list_t *host_list)
+{
+   char *host = test_framework_get_host ();
+   uint16_t port = test_framework_get_port ();
+   char *host_and_port = bson_strdup_printf ("%s:%hu", host, port);
+
+   _mongoc_host_list_from_string (host_list, host_and_port);
+
+   bson_free (host_and_port);
+   bson_free (host);
+}
+
 /*
  *--------------------------------------------------------------------------
  *
@@ -392,7 +662,7 @@ test_framework_get_user_password (char **user,
       abort ();
    }
 
-#ifndef MONGOC_ENABLE_SSL
+#ifndef MONGOC_ENABLE_CRYPTO
    if (*user && *password) {
       fprintf (stderr, "You need to configure with --enable-ssl"
                        " when providing user+password (for SCRAM-SHA-1)\n");
@@ -533,7 +803,7 @@ test_framework_get_unix_domain_socket_uri_str ()
    char *test_uri_str;
    char *test_uri_str_auth;
 
-   path = test_framework_get_unix_domain_socket_path ();
+   path = test_framework_get_unix_domain_socket_path_escaped ();
    test_uri_str = bson_strdup_printf (
       "mongodb://%s/%s",
       path,
@@ -580,10 +850,12 @@ call_ismaster_with_host_and_port (char *host,
    assert (uri);
    mongoc_uri_set_option_as_int32 (uri, "connectTimeoutMS", 10000);
    mongoc_uri_set_option_as_int32 (uri, "serverSelectionTimeoutMS", 10000);
-   mongoc_uri_set_option_as_bool (uri, "serverSelectionTryOnce", false);
+   mongoc_uri_set_option_as_bool (uri, "serverSelectionTryOnce", true);
 
    client = mongoc_client_new_from_uri (uri);
+#ifdef MONGOC_ENABLE_SSL
    test_framework_set_ssl_opts (client);
+#endif
 
    if (!mongoc_client_command_simple (client, "admin",
                                       tmp_bson ("{'isMaster': 1}"),
@@ -665,6 +937,7 @@ set_name (bson_t *ismaster_response)
 char *
 test_framework_get_uri_str_no_auth (const char *database_name)
 {
+   char *env_uri_str;
    bson_t ismaster_response;
    bson_string_t *uri_string;
    char *name;
@@ -673,57 +946,72 @@ test_framework_get_uri_str_no_auth (const char *database_name)
    bool first;
    char *host;
    uint16_t port;
+   
+   env_uri_str = _uri_str_from_env ();
+   if (env_uri_str) {
+      uri_string = bson_string_new (env_uri_str);
+      if (database_name) {
+         if (uri_string->str[uri_string->len-1] != '/') {
+            bson_string_append (uri_string, "/");
+         }
+         bson_string_append (uri_string, database_name);
+      }
+      bson_free (env_uri_str);
+   } else {
+      /* construct a direct connection or replica set connection URI */
+      call_ismaster (&ismaster_response);
+      uri_string = bson_string_new ("mongodb://");
 
-   call_ismaster (&ismaster_response);
-   uri_string = bson_string_new ("mongodb://");
+      if ((name = set_name (&ismaster_response))) {
+         /* make a replica set URI */
+         bson_iter_init_find (&iter, &ismaster_response, "hosts");
+         bson_iter_recurse (&iter, &hosts_iter);
+         first = true;
 
-   if ((name = set_name (&ismaster_response))) {
-      /* make a replica set URI */
-      bson_iter_init_find (&iter, &ismaster_response, "hosts");
-      bson_iter_recurse (&iter, &hosts_iter);
-      first = true;
+         /* append "host1,host2,host3" */
+         while (bson_iter_next (&hosts_iter)) {
+            assert (BSON_ITER_HOLDS_UTF8 (&hosts_iter));
+            if (!first) {
+               bson_string_append (uri_string, ",");
+            }
 
-      /* append "host1,host2,host3" */
-      while (bson_iter_next (&hosts_iter)) {
-         assert (BSON_ITER_HOLDS_UTF8 (&hosts_iter));
-         if (!first) {
-            bson_string_append (uri_string, ",");
+            bson_string_append (uri_string, bson_iter_utf8 (&hosts_iter, NULL));
+            first = false;
          }
 
-         bson_string_append (uri_string, bson_iter_utf8 (&hosts_iter, NULL));
-         first = false;
-      }
+         bson_string_append (uri_string, "/");
+         if (database_name) {
+            bson_string_append (uri_string, database_name);
+         }
 
-      bson_string_append (uri_string, "/");
-      if (database_name) {
-         bson_string_append (uri_string, database_name);
-      }
-
-      bson_string_append_printf (uri_string, "?replicaSet=%s", name);
-      bson_free (name);
-   } else {
-      host = test_framework_get_host ();
-      port = test_framework_get_port ();
-      bson_string_append_printf (uri_string, "%s:%hu", host, port);
-      bson_string_append (uri_string, "/");
-      if (database_name) {
-         bson_string_append (uri_string, database_name);
-      }
-
-      bson_free (host);
-   }
-
-   if (test_framework_get_ssl ()) {
-      if (name) {
-         /* string ends with "?replicaSet=name" */
-         bson_string_append (uri_string, "&ssl=true");
+         bson_string_append_printf (uri_string, "?replicaSet=%s", name);
+         bson_free (name);
       } else {
-         /* string ends with "/" or "/dbname" */
-         bson_string_append (uri_string, "?ssl=true");
+         host = test_framework_get_host ();
+         port = test_framework_get_port ();
+         bson_string_append_printf (uri_string, "%s:%hu", host, port);
+         bson_string_append (uri_string, "/");
+         if (database_name) {
+            bson_string_append (uri_string, database_name);
+         }
+
+         bson_free (host);
       }
+
+      if (test_framework_get_ssl ()) {
+         if (strchr (uri_string->str, '?')) {
+            /* string ends with "?replicaSet=name" */
+            bson_string_append (uri_string, "&ssl=true");
+         } else {
+            /* string ends with "/" or "/dbname" */
+            bson_string_append (uri_string, "?ssl=true");
+         }
+      }
+
+      bson_destroy (&ismaster_response);
    }
 
-   bson_destroy (&ismaster_response);
+
 
    return bson_string_free (uri_string, false);
 }
@@ -788,6 +1076,107 @@ test_framework_get_uri ()
    return uri;
 }
 
+
+/*
+ *--------------------------------------------------------------------------
+ *
+ * test_framework_mongos_count --
+ *
+ *       Returns the number of servers in the test framework's MongoDB URI.
+ *
+ *--------------------------------------------------------------------------
+ */
+
+size_t
+test_framework_mongos_count (void)
+{
+   mongoc_uri_t *uri = test_framework_get_uri ();
+   const mongoc_host_list_t *h;
+   size_t count = 0;
+
+   assert (uri);
+   h = mongoc_uri_get_hosts (uri);
+   while (h) {
+      ++count;
+      h = h->next;
+   }
+
+   mongoc_uri_destroy (uri);
+
+   return count;
+}
+
+
+/*
+ *--------------------------------------------------------------------------
+ *
+ * test_framework_replset_member_count --
+ *
+ *       Returns the number replica set data members (including arbiters).
+ *
+ *--------------------------------------------------------------------------
+ */
+
+size_t
+test_framework_replset_member_count (void)
+{
+   mongoc_client_t *client;
+   bson_t reply;
+   bson_error_t error;
+   bool r;
+   bson_iter_t iter, array;
+   size_t count = 0;
+
+   client = test_framework_client_new ();
+   r = mongoc_client_command_simple (client, "admin",
+                                     tmp_bson ("{'replSetGetStatus': 1}"), NULL,
+                                     &reply, &error);
+
+   if (r) {
+      if (bson_iter_init_find (&iter, &reply, "members") &&
+          BSON_ITER_HOLDS_ARRAY (&iter)) {
+         bson_iter_recurse (&iter, &array);
+         while (bson_iter_next (&array)) {
+            ++count;
+         }
+      }
+   } else if (!strstr (error.message, "not running with --replSet") &&
+              !strstr (error.message, "replSetGetStatus is not supported through mongos"))
+   {
+      /* failed for some other reason */
+      ASSERT_OR_PRINT (false, error);
+   }
+
+   bson_destroy (&reply);
+   mongoc_client_destroy (client);
+
+   return count;
+}
+
+
+/*
+ *--------------------------------------------------------------------------
+ *
+ * test_framework_server_count --
+ *
+ *       Returns the number of mongos servers or replica set members,
+ *       or 1 if the server is standalone.
+ *
+ *--------------------------------------------------------------------------
+ */
+size_t
+test_framework_server_count (void)
+{
+   size_t count = 0;
+
+   count = test_framework_replset_member_count ();
+   if (count > 0) {
+      return count;
+   }
+
+   return test_framework_mongos_count ();
+}
+
 /*
  *--------------------------------------------------------------------------
  *
@@ -984,21 +1373,11 @@ test_framework_is_mongos (void)
    return is_mongos;
 }
 
+
 bool
 test_framework_is_replset (void)
 {
-   bson_t reply;
-   bson_iter_t iter;
-   bool is_replset;
-
-   call_ismaster (&reply);
-
-   is_replset = (bson_iter_init_find (&iter, &reply, "hosts") &&
-                 BSON_ITER_HOLDS_ARRAY (&iter));
-
-   bson_destroy (&reply);
-
-   return is_replset;
+   return test_framework_replset_member_count() > 0;
 }
 
 bool
@@ -1024,6 +1403,42 @@ test_framework_server_is_secondary (mongoc_client_t *client,
    return ret;
 }
 
+
+int test_framework_skip_if_no_auth (void)
+{
+   char *user;
+
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
+
+#ifndef MONGOC_ENABLE_SSL_OPENSSL
+   if (test_framework_max_wire_version_at_least (3)) {
+      /* requires SSL for SCRAM implementation, can't test auth */
+      return 0;
+   }
+#endif
+
+   /* run auth tests if the MONGOC_TEST_USER env var is set */
+   user = test_framework_get_admin_user ();
+   bson_free (user);
+   return user ? 1 : 0;
+}
+
+
+int test_framework_skip_if_offline (void)
+{
+   return test_framework_getenv_bool ("MONGOC_TEST_OFFLINE") ? 0 : 1;
+}
+
+
+int
+test_framework_skip_if_slow (void)
+{
+   return test_framework_getenv_bool ("MONGOC_TEST_SKIP_SLOW") ? 0 : 1;
+}
+
+
 int
 test_framework_skip_if_windows (void)
 {
@@ -1034,6 +1449,31 @@ test_framework_skip_if_windows (void)
 #endif
 }
 
+
+/* skip if no Unix domain socket */
+int
+test_framework_skip_if_no_uds (void)
+{
+#ifdef _WIN32
+   return 0;
+#else
+   char *path;
+   int ret;
+
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
+
+   path = test_framework_get_unix_domain_socket_path ();
+   ret = access (path, R_OK|W_OK) == 0 ? 1 : 0;
+
+   bson_free (path);
+
+   return ret;
+#endif
+}
+
+
 bool
 test_framework_max_wire_version_at_least (int version)
 {
@@ -1170,50 +1610,108 @@ test_version_cmp (void)
 int
 test_framework_skip_if_single (void)
 {
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
    return (test_framework_is_mongos () || test_framework_is_replset());
 }
 
 int
 test_framework_skip_if_mongos (void)
 {
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
    return test_framework_is_mongos() ? 0 : 1;
 }
 
 int
 test_framework_skip_if_replset (void)
 {
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
    return test_framework_is_replset() ? 0 : 1;
 }
 
 int
 test_framework_skip_if_not_single (void)
 {
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
    return !test_framework_skip_if_single ();
 }
 
 int
 test_framework_skip_if_not_mongos (void)
 {
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
    return !test_framework_skip_if_mongos ();
 }
 
 int
 test_framework_skip_if_not_replset (void)
 {
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
    return !test_framework_skip_if_replset ();
 }
 
+int test_framework_skip_if_max_version_version_less_than_2 (void)
+{
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
+   return test_framework_max_wire_version_at_least (2);
+}
+
 int test_framework_skip_if_max_version_version_less_than_4 (void)
 {
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
    return test_framework_max_wire_version_at_least (4);
 }
 
-
-int test_framework_skip_if_offline (void)
+int test_framework_skip_if_max_version_version_more_than_4 (void)
 {
-   return test_framework_getenv_bool ("MONGOC_TEST_OFFLINE") ? 0 : 1;
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
+   return test_framework_max_wire_version_at_least (4) ? 0 : 1;
 }
 
+int test_framework_skip_if_max_version_version_less_than_5 (void)
+{
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
+   return test_framework_max_wire_version_at_least (5);
+}
+
+int test_framework_skip_if_not_rs_version_5 (void)
+{
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
+   return (test_framework_max_wire_version_at_least (5) &&
+           test_framework_is_replset ()) ? 1 : 0;
+}
+
+int test_framework_skip_if_rs_version_5 (void)
+{
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
+   return (test_framework_max_wire_version_at_least (5) &&
+           test_framework_is_replset ()) ? 0 : 1;
+}
+
+static char MONGOC_TEST_UNIQUE [32];
 
 int
 main (int   argc,
@@ -1228,7 +1726,9 @@ main (int   argc,
                   "test_%u_%u", (unsigned)time (NULL),
                   (unsigned)gettestpid ());
 
-   mongoc_log_set_handler (log_handler, NULL);
+   mongoc_mutex_init (&captured_logs_mutex);
+   _mongoc_array_init (&captured_logs, sizeof (log_entry_t *));
+   mongoc_log_set_handler (log_handler, (void *) &suite);
 
 #ifdef MONGOC_ENABLE_SSL
    test_framework_global_ssl_opts_init ();
@@ -1242,25 +1742,35 @@ main (int   argc,
    test_async_install (&suite);
    test_buffer_install (&suite);
    test_client_install (&suite);
+#ifdef MONGOC_EXPERIMENTAL_FEATURES
+   test_client_max_staleness_install (&suite);
+#endif
    test_client_pool_install (&suite);
    test_write_command_install (&suite);
    test_bulk_install (&suite);
    test_cluster_install (&suite);
    test_collection_install (&suite);
    test_collection_find_install (&suite);
+   test_command_monitoring_install (&suite);
    test_cursor_install (&suite);
    test_database_install (&suite);
+   test_error_install (&suite);
    test_exhaust_install (&suite);
    test_find_and_modify_install (&suite);
    test_gridfs_install (&suite);
    test_gridfs_file_page_install (&suite);
+#ifdef MONGOC_EXPERIMENTAL_FEATURES
+   test_linux_distro_scanner_install (&suite);
+#endif
    test_list_install (&suite);
    test_log_install (&suite);
    test_matcher_install (&suite);
+#ifdef MONGOC_EXPERIMENTAL_FEATURES
+   test_metadata_install (&suite);
+#endif
    test_queue_install (&suite);
    test_read_prefs_install (&suite);
    test_rpc_install (&suite);
-   test_sasl_install (&suite);
    test_socket_install (&suite);
    test_topology_scanner_install (&suite);
    test_topology_reconcile_install (&suite);
@@ -1273,18 +1783,24 @@ main (int   argc,
    test_topology_install (&suite);
    test_uri_install (&suite);
    test_usleep_install (&suite);
+   test_util_install (&suite);
    test_version_install (&suite);
    test_write_concern_install (&suite);
 #ifdef MONGOC_ENABLE_SSL
-   test_x509_install (&suite);
    test_stream_tls_install (&suite);
+   test_x509_install (&suite);
    test_stream_tls_error_install (&suite);
 #endif
+#ifdef MONGOC_ENABLE_SASL
+   test_sasl_install (&suite);
+#endif
 
    ret = TestSuite_Run (&suite);
 
    TestSuite_Destroy (&suite);
 
+   capture_logs (false);  /* clear entries */
+   _mongoc_array_destroy (&captured_logs);
    mongoc_cleanup();
 
    return ret;
