@@ -7,7 +7,6 @@
 #include "mock_server/mock-server.h"
 #include "mock_server/future.h"
 #include "mock_server/future-functions.h"
-#include "mongoc-tests.h"
 #include "TestSuite.h"
 #include "test-libmongoc.h"
 #include "test-conveniences.h"
@@ -151,6 +150,8 @@ _test_cluster_node_disconnect (bool pooled)
    uint16_t client_port_0, client_port_1;
    bson_error_t error;
 
+   capture_logs (true);
+
    server = mock_server_with_autoismaster (0);
    mock_server_run (server);
 
@@ -168,9 +169,6 @@ _test_cluster_node_disconnect (bool pooled)
 
    /* query 0 fails. set client_port_0 to the port used by the query. */
    START_QUERY (client_port_0);
-   if (pooled) {
-      suppress_one_message ();
-   }
 
    mock_server_resets (request);
    ASSERT_CURSOR_ERR ();
@@ -200,14 +198,14 @@ _test_cluster_node_disconnect (bool pooled)
 
 
 static void
-test_cluster_node_disconnect_single (void)
+test_cluster_node_disconnect_single (void *ctx)
 {
    _test_cluster_node_disconnect (false);
 }
 
 
 static void
-test_cluster_node_disconnect_pooled (void)
+test_cluster_node_disconnect_pooled (void *ctx)
 {
    _test_cluster_node_disconnect (true);
 }
@@ -226,6 +224,8 @@ _test_cluster_command_timeout (bool pooled)
    uint16_t client_port;
    bson_t reply;
 
+   capture_logs (true);
+
    server = mock_server_with_autoismaster (0);
    mock_server_run (server);
    uri = mongoc_uri_copy (mock_server_get_uri (server));
@@ -239,7 +239,6 @@ _test_cluster_command_timeout (bool pooled)
    }
 
    /* server doesn't respond in time */
-   suppress_one_message ();
    future = future_client_command_simple (client, "db", tmp_bson ("{'foo': 1}"),
                                           NULL, NULL, &error);
    request = mock_server_receives_command (server, "db", MONGOC_QUERY_SLAVE_OK,
@@ -251,8 +250,6 @@ _test_cluster_command_timeout (bool pooled)
                           MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET,
                           "Failed to send \"foo\" command with database \"db\"");
 
-   /* late response */
-   mock_server_replies_simple (request, "{'ok': 1, 'bar': 1}");
    request_destroy (request);
    future_destroy (future);
 
@@ -260,6 +257,7 @@ _test_cluster_command_timeout (bool pooled)
                                           NULL, &reply, &error);
    request = mock_server_receives_command (server, "db", MONGOC_QUERY_SLAVE_OK,
                                            "{'baz': 1}");
+   ASSERT (request);
    /* new socket */
    ASSERT_CMPUINT16 (client_port, !=, request_get_client_port (request));
    mock_server_replies_simple (request, "{'ok': 1, 'quux': 1}");
@@ -369,14 +367,14 @@ _test_write_disconnect (bool legacy)
 
 
 static void
-test_write_command_disconnect (void)
+test_write_command_disconnect (void *ctx)
 {
    _test_write_disconnect (false);
 }
 
 
 static void
-test_legacy_write_disconnect (void)
+test_legacy_write_disconnect (void *ctx)
 {
    _test_write_disconnect (true);
 }
@@ -474,14 +472,14 @@ test_legacy_write_socket_check (void)
 void
 test_cluster_install (TestSuite *suite)
 {
-   TestSuite_Add (suite, "/Cluster/test_get_max_bson_obj_size", test_get_max_bson_obj_size);
-   TestSuite_Add (suite, "/Cluster/test_get_max_msg_size", test_get_max_msg_size);
-   TestSuite_Add (suite, "/Cluster/disconnect/single", test_cluster_node_disconnect_single);
-   TestSuite_Add (suite, "/Cluster/disconnect/pooled", test_cluster_node_disconnect_pooled);
+   TestSuite_AddLive (suite, "/Cluster/test_get_max_bson_obj_size", test_get_max_bson_obj_size);
+   TestSuite_AddLive (suite, "/Cluster/test_get_max_msg_size", test_get_max_msg_size);
+   TestSuite_AddFull  (suite, "/Cluster/disconnect/single", test_cluster_node_disconnect_single, NULL, NULL, test_framework_skip_if_slow);
+   TestSuite_AddFull  (suite, "/Cluster/disconnect/pooled", test_cluster_node_disconnect_pooled, NULL, NULL, test_framework_skip_if_slow);
    TestSuite_Add (suite, "/Cluster/command/timeout/single", test_cluster_command_timeout_single);
    TestSuite_Add (suite, "/Cluster/command/timeout/pooled", test_cluster_command_timeout_pooled);
-   TestSuite_Add (suite, "/Cluster/write_command/disconnect", test_write_command_disconnect);
-   TestSuite_Add (suite, "/Cluster/legacy_write/disconnect", test_legacy_write_disconnect);
+   TestSuite_AddFull (suite, "/Cluster/write_command/disconnect", test_write_command_disconnect, NULL, NULL, test_framework_skip_if_slow);
+   TestSuite_AddFull  (suite, "/Cluster/legacy_write/disconnect", test_legacy_write_disconnect, NULL, NULL, test_framework_skip_if_slow);
    TestSuite_Add (suite, "/Cluster/write_command/socket_check", test_write_command_socket_check);
    TestSuite_Add (suite, "/Cluster/legacy_write/socket_check", test_legacy_write_socket_check);
 }
