@@ -8,11 +8,6 @@
 
 #include "test-libmongoc.h"
 
-#define ASSERT_SUPPRESS(x) \
-   do { \
-      suppress_one_message (); \
-      ASSERT (x); \
-   } while (0)
 
 static void
 test_mongoc_uri_new (void)
@@ -27,24 +22,26 @@ test_mongoc_uri_new (void)
    bson_iter_t iter;
    bson_iter_t child;
 
+   capture_logs (true);
+
    /* bad uris */
    ASSERT(!mongoc_uri_new("mongodb://"));
-   ASSERT_SUPPRESS(!mongoc_uri_new("mongodb://\x80"));
-   ASSERT_SUPPRESS(!mongoc_uri_new("mongodb://localhost/\x80"));
-   ASSERT_SUPPRESS(!mongoc_uri_new("mongodb://localhost:\x80/"));
-   ASSERT_SUPPRESS(!mongoc_uri_new("mongodb://localhost/?ipv6=\x80"));
-   ASSERT_SUPPRESS(!mongoc_uri_new("mongodb://localhost/?foo=\x80"));
-   ASSERT_SUPPRESS(!mongoc_uri_new("mongodb://localhost/?\x80=bar"));
-   ASSERT_SUPPRESS(!mongoc_uri_new("mongodb://\x80:pass@localhost"));
-   ASSERT_SUPPRESS(!mongoc_uri_new("mongodb://user:\x80@localhost"));
-   ASSERT_SUPPRESS(!mongoc_uri_new("mongodb://user%40DOMAIN.COM:password@localhost/?"
-                                   "authMechanism=\x80"));
-   ASSERT_SUPPRESS(!mongoc_uri_new("mongodb://user%40DOMAIN.COM:password@localhost/?"
-                                   "authMechanism=GSSAPI"
-                                   "&authMechanismProperties=SERVICE_NAME:\x80"));
-   ASSERT_SUPPRESS(!mongoc_uri_new("mongodb://user%40DOMAIN.COM:password@localhost/?"
-                                   "authMechanism=GSSAPI"
-                                   "&authMechanismProperties=\x80:mongodb"));
+   ASSERT(!mongoc_uri_new("mongodb://\x80"));
+   ASSERT(!mongoc_uri_new("mongodb://localhost/\x80"));
+   ASSERT(!mongoc_uri_new("mongodb://localhost:\x80/"));
+   ASSERT(!mongoc_uri_new("mongodb://localhost/?ipv6=\x80"));
+   ASSERT(!mongoc_uri_new("mongodb://localhost/?foo=\x80"));
+   ASSERT(!mongoc_uri_new("mongodb://localhost/?\x80=bar"));
+   ASSERT(!mongoc_uri_new("mongodb://\x80:pass@localhost"));
+   ASSERT(!mongoc_uri_new("mongodb://user:\x80@localhost"));
+   ASSERT(!mongoc_uri_new("mongodb://user%40DOMAIN.COM:password@localhost/?"
+                          "authMechanism=\x80"));
+   ASSERT(!mongoc_uri_new("mongodb://user%40DOMAIN.COM:password@localhost/?"
+                          "authMechanism=GSSAPI"
+                          "&authMechanismProperties=SERVICE_NAME:\x80"));
+   ASSERT(!mongoc_uri_new("mongodb://user%40DOMAIN.COM:password@localhost/?"
+                          "authMechanism=GSSAPI"
+                          "&authMechanismProperties=\x80:mongodb"));
    ASSERT(!mongoc_uri_new("mongodb://::"));
    ASSERT(!mongoc_uri_new("mongodb://localhost::27017"));
    ASSERT(!mongoc_uri_new("mongodb://localhost,localhost::"));
@@ -415,7 +412,11 @@ test_mongoc_uri_functions (void)
    mongoc_client_destroy (client);
 
 
-   uri = mongoc_uri_new("mongodb://localhost/?serverselectiontimeoutms=3&journal=true&wtimeoutms=42&canonicalizeHostname=false");
+   uri = mongoc_uri_new("mongodb://localhost/?serverselectiontimeoutms=3"
+                           "&journal=true"
+                           "&wtimeoutms=42"
+                           "&localthresholdms=17"
+                           "&canonicalizeHostname=false");
 
    ASSERT_CMPINT(mongoc_uri_get_option_as_int32(uri, "serverselectiontimeoutms", 18), ==, 3);
    ASSERT(mongoc_uri_set_option_as_int32(uri, "serverselectiontimeoutms", 18));
@@ -425,6 +426,10 @@ test_mongoc_uri_functions (void)
    ASSERT(mongoc_uri_set_option_as_int32(uri, "wtimeoutms", 18));
    ASSERT_CMPINT(mongoc_uri_get_option_as_int32(uri, "wtimeoutms", 19), ==, 18);
 
+   ASSERT_CMPINT(mongoc_uri_get_option_as_int32(uri, "localthresholdms", 99), ==, 17);
+   ASSERT(mongoc_uri_set_option_as_int32(uri, "localthresholdms", 99));
+   ASSERT_CMPINT(mongoc_uri_get_option_as_int32(uri, "localthresholdms", 42), ==, 99);
+
    /* socketcheckintervalms isn't set, return our fallback */
    ASSERT_CMPINT(mongoc_uri_get_option_as_int32(uri, "socketcheckintervalms", 123), ==, 123);
    ASSERT(mongoc_uri_set_option_as_int32(uri, "socketcheckintervalms", 18));
@@ -486,6 +491,49 @@ test_mongoc_uri_functions (void)
 
 #undef ASSERT_SUPPRESS
 
+
+static void
+test_mongoc_uri_compound_setters (void)
+{
+   mongoc_uri_t *uri;
+   mongoc_read_prefs_t *prefs;
+   const mongoc_read_prefs_t *prefs_result;
+   mongoc_read_concern_t *rc;
+   const mongoc_read_concern_t *rc_result;
+   mongoc_write_concern_t *wc;
+   const mongoc_write_concern_t *wc_result;
+
+   uri = mongoc_uri_new ("mongodb://localhost/"
+                            "?readPreference=nearest&readPreferenceTags=dc:ny"
+                            "&readConcern=majority"
+                            "&w=3");
+
+   prefs = mongoc_read_prefs_new (MONGOC_READ_SECONDARY);
+   mongoc_uri_set_read_prefs_t (uri, prefs);
+   prefs_result = mongoc_uri_get_read_prefs_t (uri);
+   ASSERT_CMPINT (mongoc_read_prefs_get_mode (prefs_result),
+                  ==, MONGOC_READ_SECONDARY);
+      ASSERT (bson_empty (mongoc_read_prefs_get_tags (prefs_result)));
+
+   rc = mongoc_read_concern_new ();
+   mongoc_read_concern_set_level (rc, "whatever");
+   mongoc_uri_set_read_concern (uri, rc);
+   rc_result = mongoc_uri_get_read_concern (uri);
+   ASSERT_CMPSTR (mongoc_read_concern_get_level (rc_result), "whatever");
+
+   wc = mongoc_write_concern_new ();
+   mongoc_write_concern_set_w (wc, 2);
+   mongoc_uri_set_write_concern (uri, wc);
+   wc_result = mongoc_uri_get_write_concern (uri);
+   ASSERT_CMPINT32 (mongoc_write_concern_get_w (wc_result), ==, (int32_t) 2);
+
+   mongoc_read_prefs_destroy (prefs);
+   mongoc_read_concern_destroy (rc);
+   mongoc_write_concern_destroy (wc);
+   mongoc_uri_destroy (uri);
+}
+
+
 static void
 test_mongoc_host_list_from_string (void)
 {
@@ -608,11 +656,16 @@ test_mongoc_uri_read_prefs (void)
    for (i = 0; tests[i].uri; i++) {
       t = &tests[i];
 
+      capture_logs (true);
       uri = mongoc_uri_new(t->uri);
       if (t->parses) {
          assert(uri);
+         ASSERT_NO_CAPTURED_LOGS (t->uri);
       } else {
          assert(!uri);
+         ASSERT_CAPTURED_LOG (
+            t->uri, MONGOC_LOG_LEVEL_WARNING,
+            "Primary read preference mode conflicts with tags");
          continue;
       }
 
@@ -642,6 +695,7 @@ typedef struct
    int32_t     w;
    const char *wtag;
    int32_t     wtimeoutms;
+   const char *log_msg;
 } write_concern_test;
 
 
@@ -666,23 +720,34 @@ test_mongoc_uri_write_concern (void)
       { "mongodb://localhost/?w=mytag&safe=false", true, MONGOC_WRITE_CONCERN_W_TAG, "mytag" },
       { "mongodb://localhost/?w=1&safe=false", true, 1 },
       { "mongodb://localhost/?journal=true", true, MONGOC_WRITE_CONCERN_W_DEFAULT },
-      { "mongodb://localhost/?w=0&journal=true", false, MONGOC_WRITE_CONCERN_W_UNACKNOWLEDGED },
-      { "mongodb://localhost/?w=-1&journal=true", false, MONGOC_WRITE_CONCERN_W_ERRORS_IGNORED },
       { "mongodb://localhost/?w=1&journal=true", true, 1 },
       { "mongodb://localhost/?w=2&wtimeoutms=1000", true, 2, NULL, 1000 },
       { "mongodb://localhost/?w=majority&wtimeoutms=1000", true, MONGOC_WRITE_CONCERN_W_MAJORITY, NULL, 1000 },
       { "mongodb://localhost/?w=mytag&wtimeoutms=1000", true, MONGOC_WRITE_CONCERN_W_TAG, "mytag", 1000 },
+      { "mongodb://localhost/?w=0&journal=true", false, MONGOC_WRITE_CONCERN_W_UNACKNOWLEDGED,
+        NULL, 0, "Journal conflicts with w value [w=0]" },
+      { "mongodb://localhost/?w=-1&journal=true",          false,
+        MONGOC_WRITE_CONCERN_W_ERRORS_IGNORED,
+        NULL, 0, "Journal conflicts with w value [w=-1]" },
       { NULL }
    };
 
-   /* Suppress warnings from two invalid URIs ("journal" and "w" conflict) */
-   suppress_one_message();
-   suppress_one_message();
-
    for (i = 0; tests [i].uri; i++) {
       t = &tests [i];
 
+      capture_logs (true);
       uri = mongoc_uri_new (t->uri);
+
+      if (tests [i].log_msg) {
+         ASSERT_CAPTURED_LOG (tests [i].uri,
+                              MONGOC_LOG_LEVEL_WARNING,
+                              tests [i].log_msg);
+      } else {
+         ASSERT_NO_CAPTURED_LOGS (tests [i].uri);
+      }
+
+      capture_logs (false);  /* clear captured logs */
+
       if (t->parses) {
          assert (uri);
       } else {
@@ -754,7 +819,6 @@ test_mongoc_uri_read_concern (void)
 }
 
 
-
 void
 test_uri_install (TestSuite *suite)
 {
@@ -766,4 +830,5 @@ test_uri_install (TestSuite *suite)
    TestSuite_Add (suite, "/Uri/write_concern", test_mongoc_uri_write_concern);
    TestSuite_Add (suite, "/HostList/from_string", test_mongoc_host_list_from_string);
    TestSuite_Add (suite, "/Uri/functions", test_mongoc_uri_functions);
+   TestSuite_Add (suite, "/Uri/compound_setters", test_mongoc_uri_compound_setters);
 }
